{% extends 'lara_django_base/lara_base.html' %}
{% block toolbar %}
    <div class="d-flex justify-content-between flex-wrap flex-md-nowrap align-items-center pt-3 pb-2 mb-3 border-bottom">
        <h1 class="h2">{{ section_title }}</h1>
        <div class="btn-toolbar mb-2 mb-md-0">
            <div class="btn-group me-2">
                <a href="{% url update_link  object.pk %}"
                   class="btn btn-warning"
                   role="button">Edit Item</a>
            </div>
        </div>
    </div>
{% endblock %}
{% block content %}
    <h1>
        {{ object.name_full }}
        {% if object.category %}({{ object.category }}){% endif %}
    </h1>
    <div>
        <p>
            Product Type:
            <span style="font-weight: bold;">{{ object.product_type }}</span>
        </p>
        <p>
            Name:
            <span style="font-weight: bold;">{{ object.name_full }}</span>
        </p>
        <p>
            Manufacturer:
            <span style="font-weight: bold;">{{ object.manufacturer.name }}</span>
        </p>
    </div>
    <hr />
    <h4>Model Details</h4>
    <p>
        Model no.:
        <span style="font-weight: bold;">{{ object.model_no }}</span>
    </p>
    <p>
        Prod. no.:
        <span style="font-weight: bold;">{{ object.product_no }}</span>
    </p>
    <p>
        Quickstart:
        <a style="font-weight: bold;" href="{{ object.maunal }}">{{ object.manual }}</a>
    </p>
    <p>
        Manual:
        <a style="font-weight: bold;" href="{{ object.maunal }}">{{ object.manual }}</a>
    </p>
    <p>
        Service Manual:
        <a style="font-weight: bold;" href="{{ object.service_maunal }}">{{ object.service_manual }}</a>
    </p>
    <p>
        URL:
        <a style="font-weight: bold;" href="{{ object.URL }}">{{ object.URL }}</a>
    </p>
    <h4>Specifications</h4>
    <div>
        <p>
            Specifications:
            <span style="font-weight: bold;">{{ object.specifications }}</span>
        </p>
        <p>
            Specifications URL:
            <a style="font-weight: bold;" href="{{ object.spec_doc }}">{{ object.URL }}</a>
        </p>
        <p>
            Energy Consumption:
            <span style="font-weight: bold;">{{ object.energy_consumption }}</span>
        </p>
    </div>
{% endblock %}
