{% extends "base.html" %}
{% block title %}Search People{% endblock %}
{% block top %}
    <h1>Search LARA People</h1>
    <p></p>
{% endblock %}
{% block main_content %}
    <center>
        <form action="{% url 'lara_django_people:search' %}" method="post">
            {% csrf_token %}
            {# <label for="input_search">Search person: </label> #}
            <input type="text" name="search" id="input_search" value="" />
            <input type="submit" value="Search" />
        </form>
    </center>
{% endblock %}
